import { EntityTypes, ItemTypes, BlockTypes, ItemStack, system, world } from "@minecraft/server";
import { ActionFormData, MessageFormData, ModalFormData } from "@minecraft/server-ui";
import * as UTIL from "./utils";

// giftListJson = {
//   items : [
//     { 
//       itemid: "",
//       qty: "",
//       modname: "",
//       modlore: ""
//     }
//   ]
// }
const ERRMSG = "§c【エラー】§r";

world.beforeEvents.itemUse.subscribe((ev) => {
  const player = ev.source;
  if (ev.itemStack.typeId !== "gift:gift_box") return;
  if (ev.source.typeId !== EntityTypes.get("minecraft:player").id) return;

  let initFlg = world.getDynamicProperty(`loginBonusInitFlg_${player.name}`);
  if (initFlg === undefined) initFlg = true;
  const initGiftItemsDymcs = world.getDynamicProperty(`loginBonusInitItems`);
  if (!initGiftItemsDymcs) initFlg = false;

  system.run(() => {
    const mainForm = new ActionFormData();
    mainForm.title(`ギフトボックス`);
    mainForm.body(`ログインボーナスを受け取れます`);

    const JST = UTIL.dateNow();
    const JSTdate = JST.getDate();
    let accepted = "";
    if (chkDay(JSTdate, player)) { accepted = "\n§t***本日受取済み***"; }
    mainForm.button(`もらう！${accepted}`, "textures/items/promo_gift_small_blue");
    if (initFlg && initGiftItemsDymcs) {
      mainForm.button(`§4初回限定§rボーナス！\n一度だけ受領可能！`, "textures/items/promo_gift_small_blue");
    }
    if (player.isOp() || player.hasTag("op")) {
      mainForm.button(`ワールド管理者用設定`, "textures/ui/gear");
    }

    mainForm.show(player).then(response => {
      if (response.canceled) return;
      if (response.selection === 0) { giveLoginBonus(player, JSTdate) }
      else if (initFlg && response.selection === 1) { giveInitLoginBonus(player) }
      else if (initFlg && response.selection === 2) { showManagement(player) }
      else if (!initFlg && response.selection === 1) { showManagement(player) };
    });
  });
});

/*** 初回限定ログボ ***/
function giveInitLoginBonus(player){

  const initGiftItemsDymcs = world.getDynamicProperty(`loginBonusInitItems`);
    if (!initGiftItemsDymcs) {
    player.sendMessage(`§eアイテムが未設定です！ワールド管理者（オペレーター）による設定が必要です。`);
    return;
  }
  const initGiftItemsDymcsJson = JSON.parse(world.getDynamicProperty(`loginBonusInitItems`));

  let giveInitItem = initGiftItemsDymcsJson.items[0].itemid;
  let giveInitQuantity = initGiftItemsDymcsJson.items[0].qty;
  let giveInitItemTag = initGiftItemsDymcsJson.items[0].modname;
  let giveInitLore = initGiftItemsDymcsJson.items[0].modlore;

  if (!checkEmptySlot(player, giveInitItem, giveInitQuantity)) return;

  player.sendMessage(`§l§e｡.｡:+* ﾟ ゜ﾟ *+:｡.｡:+* ﾟ ゜ﾟ *+:｡.｡.｡:+* ﾟ`);
  player.sendMessage(`§e【初回限定】ログインボーナス！`);
  let item = new ItemStack(ItemTypes.get(addStrMine(giveInitItem)), giveInitQuantity);
  if (giveInitItemTag) item.nameTag = giveInitItemTag;
  if (giveInitLore) {
    let setLores = giveInitLore.split("@");
    item.setLore(setLores);
  }
  player.getComponent("minecraft:inventory").container.addItem(item);
  player.sendMessage({rawtext:[{text:`「${giveInitItem.replace("minecraft:", "")} ${giveInitQuantity}個」を受け取りました`}]});
  world.setDynamicProperty(`loginBonusInitFlg_${player.name}`, false);
  player.sendMessage(`§l§e｡.｡:+* ﾟ ゜ﾟ *+:｡.｡:+* ﾟ ゜ﾟ *+:｡.｡.｡:+* ﾟ`);
  player.playSound("random.levelup");
}

/*** ログボ ***/
function giveLoginBonus(player, JSTdate){
  if (chkDay(JSTdate, player)) {
    player.onScreenDisplay.setActionBar(`本日分は§l§e受取済み§rです！`);
  } else {
    let GiveItems = [];
    let GiveQuantitys = [];
    let GiveItemTags = [];
    let GiveLores = [];

    let giftItemsDymcsJson = JSON.parse(world.getDynamicProperty(`loginBonusItems`));
    let itemsLength = Object.keys(giftItemsDymcsJson.items).length;

    if (!itemsLength) {
      player.sendMessage(`§eアイテムが未設定です！ワールド管理者（オペレーター）による設定が必要です。`);
      return;
    } else {
      for (let i = 0; i < itemsLength; i++) {
        GiveItems.push(giftItemsDymcsJson.items[i].itemid);
        GiveQuantitys.push(giftItemsDymcsJson.items[i].qty);
        GiveItemTags.push(giftItemsDymcsJson.items[i].modname);
        GiveLores.push(giftItemsDymcsJson.items[i].modlore);
      }
    }

    if (!checkEmptySlot(player, GiveItems, GiveQuantitys)) return;

    player.sendMessage(`§l§e｡.｡:+* ﾟ ゜ﾟ *+:｡.｡:+* ﾟ ゜ﾟ *+:｡.｡.｡:+* ﾟ`);
    player.sendMessage(`§eログインボーナス！`);
    for (let k = 0; k < itemsLength; k++) {
      let item = new ItemStack(ItemTypes.get(addStrMine(GiveItems[k])), GiveQuantitys[k]);
      if (GiveItemTags[k]) item.nameTag = GiveItemTags[k];
      if (GiveLores[k]) {
        let setLores = GiveLores[k].split("@");
        item.setLore(setLores);
      }
      player.getComponent("minecraft:inventory").container.addItem(item);
      player.sendMessage({rawtext:[{text:`「${GiveItems[k].replace("minecraft:", "")} ${GiveQuantitys[k]}個」を受け取りました`}]});
    }
    const scrIdentLoginBonusDay = UTIL.getObjective("loginBonusDay");
    UTIL.setScore(scrIdentLoginBonusDay, player.scoreboardIdentity, JSTdate);

    player.sendMessage(`§eまた明日受け取ってね！`);
    player.sendMessage(`§l§e｡.｡:+* ﾟ ゜ﾟ *+:｡.｡:+* ﾟ ゜ﾟ *+:｡.｡.｡:+* ﾟ`);
    player.playSound("random.levelup");
  }
}
function chkDay(JSTdate, player) {
  const scrIdentLoginBonusDay = UTIL.getObjective("loginBonusDay");
  const scrloginBonusDay = UTIL.getScore(scrIdentLoginBonusDay, player.scoreboardIdentity);
  
  if (scrloginBonusDay === undefined || scrloginBonusDay === null) {
    // UTIL.setScore(scrIdentLoginBonusDay, player.scoreboardIdentity, 0); //初期時設定できない
    player.runCommandAsync(`scoreboard players set @s loginBonusDay 0`);
  }
  if (JSTdate === scrloginBonusDay) {
    return true;
  }
  return false;
}

/*** 管理者メンテナンス ***/
function showManagement(player){
  system.run(() => {
    const selectForm = new ActionFormData()
      .title(`ログボアイテム管理`)
      .body(`ログインボーナスのアイテムを管理します`)
      .button(`登録済み一覧`, "textures/ui/book_metatag_default")
      .button(`追加登録`, "textures/ui/book_edit_default")
      .button(`初回限定アイテム編集`, "textures/ui/book_edit_default");

    selectForm.show(player).then(response => {
      if (response.canceled) return;
      if (response.selection === 0) { showLoginBonusList(player) }
      else if (response.selection === 1) { 
        let initFlg = false;
        regLoginBonusItem(player, initFlg)
      }
      else if (response.selection === 2) {
        regInitLoginBonusItem(player)
      };
    });
  });
}
function showLoginBonusList(player){
  system.run(() => {
    const listForm = new ActionFormData();
    listForm.title(`アイテム一覧`);

    const giftItemsDymcs = world.getDynamicProperty(`loginBonusItems`);
    let itemsLength;

    if (!giftItemsDymcs) {
      listForm.body(`未登録`);
      listForm.button(`現在は未登録です`, "textures/ui/infobulb");
    } else {
      listForm.body(`登録済みのアイテム一覧\n§t（一部テクスチャ表示不可）`);

      let giftItemsDymcsJson = JSON.parse(world.getDynamicProperty(`loginBonusItems`));
      itemsLength = Object.keys(giftItemsDymcsJson.items).length;

      for (let i = 0; i < itemsLength; i++) {
        let item = giftItemsDymcsJson.items[i].itemid;
        let qty = giftItemsDymcsJson.items[i].qty;
        let modname = giftItemsDymcsJson.items[i].modname;
        let modlore = giftItemsDymcsJson.items[i].modlore;

        let setStr = `${item} ${qty}個\n`;
        if (modname) setStr = setStr + modname;
        if (modlore) setStr = setStr + " " + modlore;

        let iconCat = "items";
        if (BlockTypes.get(item)) iconCat = "blocks";
        let icon = "textures/" + iconCat + "/" + item.replace("minecraft:", "");
        listForm.button(`${setStr}`, icon);
      }
    }
    listForm.button(`戻る`, "textures/ui/import");

    listForm.show(player).then(response => {
      if (response.canceled) return;
      if (!itemsLength) {
        showManagement(player);
        return;
      }
      if (response.selection === itemsLength) {
        showManagement(player);
        return;
      }
      let delItemNo = response.selection;

      let giftItemsDymcsJson = JSON.parse(world.getDynamicProperty(`loginBonusItems`));
      let item = giftItemsDymcsJson.items[delItemNo].itemid;
      let qty = giftItemsDymcsJson.items[delItemNo].qty;
      let setStr = `${item} ${qty}個`;
      
      const selectForm = new MessageFormData()
      .title(`設定解除`)
      .body(`選択のアイテム「${setStr}」の登録を解除しますか`)
      .button1(`削除しない`)
      .button2(`削除！`);

      selectForm.show(player).then(response => {
        if (response.canceled) return;
        if (response.selection === 1) {
        if (itemsLength === 1) {
          world.setDynamicProperty(`loginBonusItems`, );
        } else {
          giftItemsDymcsJson.items.splice(delItemNo, 1);
          world.setDynamicProperty(`loginBonusItems`, JSON.stringify(giftItemsDymcsJson));
        }
        player.sendMessage({rawtext:[{text:`§e「${item.replace("minecraft:", "")} ${qty}個」の登録を解除しました！`}]});

        } else if (response.selection === 0) {
          showLoginBonusList(player);
          return;
        }
      });
    });
  });
}
function regLoginBonusItem(player, initFlg) {
  let regForm = new ModalFormData()
    .title("登録するアイテムを設定")
    .textField("アイテム名（ID）", "例）apple")
    .slider("数量", 1, 64, 1)
    .textField("アイテム名の変更 ※任意", "例）配布用リンゴ")
    .textField("アイテムの説明 ※任意", "例）普通のリンゴ「@」で改行");

  regForm.show(player).then(response => {
    if (response.canceled) return;
    const resItemid = response.formValues[0];
    const resQty = response.formValues[1];
    const resNameTag = response.formValues[2];
    const resLore = response.formValues[3];

    let mineStrItem = addStrMine(resItemid);

    if (!ItemTypes.get(mineStrItem)) {
      player.sendMessage(`${ERRMSG}§e一致するアイテムがありません！`);
      return;
    }
    let maxAmount = new ItemStack(mineStrItem).maxAmount;
    if (resQty > maxAmount) {
      player.sendMessage(`${ERRMSG}§e1スタック分までの数量§rにしてください！ 設定数：${resQty}`);
      player.sendMessage({rawtext:[{text:`${resItemid} の最大スタックは §e${maxAmount}§r`}]});
      return;
    }

    let items = { 
          itemid: resItemid,
          qty: resQty,
          modname: resNameTag,
          modlore: resLore
      }

    if (initFlg) {
      let initGiftItemsDymcsJson = {
        items : [items]
      };
      world.setDynamicProperty(`loginBonusInitItems`, JSON.stringify(initGiftItemsDymcsJson));
    } else {
      const giftItemsDymcs = world.getDynamicProperty(`loginBonusItems`);
      if (!giftItemsDymcs) {
        let giftItemsDymcsJson = {
          items : [items]
        };
        world.setDynamicProperty(`loginBonusItems`, JSON.stringify(giftItemsDymcsJson));
      } else {
        let giftItemsDymcsJson = JSON.parse(world.getDynamicProperty(`loginBonusItems`));
        giftItemsDymcsJson.items.push(items);
        world.setDynamicProperty(`loginBonusItems`, JSON.stringify(giftItemsDymcsJson));
      }
    }
    player.sendMessage({rawtext:[{text: resItemid + " " + resQty + "個を登録しました。"}]});
    if (resNameTag) player.sendMessage(`アイテム名変更：§e${resNameTag}§r`);
    if (resLore) player.sendMessage(`アイテム説明変更：§e${resLore}§r`);
  });
}

function regInitLoginBonusItem(player) {
  const initGiftItemsDymcs = world.getDynamicProperty(`loginBonusInitItems`);
  if (!initGiftItemsDymcs) {
    let initFlg = true;
    regLoginBonusItem(player, initFlg);
  } else {
    let initGiftItemsDymcsJson = JSON.parse(world.getDynamicProperty(`loginBonusInitItems`));
    let item = initGiftItemsDymcsJson.items[0].itemid;
    let qty = initGiftItemsDymcsJson.items[0].qty;
    let modname = initGiftItemsDymcsJson.items[0].modname;
    let modlore = initGiftItemsDymcsJson.items[0].modlore;

    let setStr = `${item} ${qty}個\n`;
    if (modname) setStr = setStr + modname;
    if (modlore) setStr = setStr + " " + modlore;

    system.run(() => {
      const initListForm = new ActionFormData();
      initListForm.title(`アイテム一覧`);
      initListForm.body(`登録済みのアイテム一覧\n※初回限定のアイテムは1種類のみ登録可能\n§t（一部テクスチャ表示不可）`);
  
      let iconCat = "items";
      if (BlockTypes.get(item)) iconCat = "blocks";
      let icon = "textures/" + iconCat + "/" + item.replace("minecraft:", "");
      initListForm.button(`${setStr}`, icon);
      initListForm.button(`初回受取りリセット`, "textures/ui/recap_glyph_desaturated");
      initListForm.button(`戻る`, "textures/ui/import");

      initListForm.show(player).then(response => {
        if (response.canceled) return;
        if (response.selection === 0) {
        const selectForm = new MessageFormData()
          .title(`設定解除`)
          .body(`選択のアイテム「${item.replace("minecraft:", "")} ${qty}個」の登録を解除しますか`)
          .button1(`削除しない`)
          .button2(`削除！`);
    
          selectForm.show(player).then(response => {
            if (response.canceled) return;
            if (response.selection === 1) {
              world.setDynamicProperty(`loginBonusInitItems`, );
              player.sendMessage({rawtext:[{text:`【初回限定アイテム編集】§e「${item.replace("minecraft:", "")} ${qty}個」の登録を解除しました！`}]});
              return;
            } else if (response.selection === 0) {
              showManagement(player);
              return;
            }
          })
        } else if (response.selection === 1) {
          const initResetForm = new MessageFormData()
          .title(`初回受取りリセット`)
          .body(`初回受取り済みをリセットして再度、初回受取りを実施可能にしますか？\n※受取り済みの全プレイヤーがリセット対象`)
          .button1(`やめる`)
          .button2(`リセット！`);
    
          initResetForm.show(player).then(response => {
            if (response.canceled) return;
            if (response.selection === 1) {
              const allInitFlgPlayerList = getAllInitFlgPlayerList();
              if (Object.keys(allInitFlgPlayerList).length  === 0) {
                player.sendMessage(`初回限定ボーナスを受取済みプレイヤーが存在しません！`);
                return;
              }
              for (let i = 0; i < allInitFlgPlayerList.length; i++) {
                world.setDynamicProperty(allInitFlgPlayerList[i], );
              }
              player.sendMessage(`すべてのプレイヤーの初回限定ボーナスが再度受取り可能になりました。`);
              return;
            }
          })
        } else if (response.selection === 2) {
          showManagement(player);
          return;
        }
      })
    })
  }
}

function addStrMine(itemName) {
  let mineStrItem = !itemName.startsWith("minecraft:") ?
  "minecraft:" + itemName : itemName;
  return mineStrItem;
}
function getAllInitFlgPlayerList() {
  const dymcsList = world.getDynamicPropertyIds();
  const dExceptList = ['loginBonusItems', 'loginBonusInitItems'];
  let allInitFlgPlayerList = dymcsList.filter(data => !dExceptList.includes(data));

  return allInitFlgPlayerList;
}

/*** インベントリ空きチェック ***/
function checkEmptySlot(player, GiveItems, GiveQuantitys) {

  const {container} = player.getComponent("minecraft:inventory");
  let addSlot = 0;

  for (let k = 0; k < GiveItems.length; k++) {
    let giveItem = "minecraf:air";
    let giveQuantity = 0;
    let EmptySlot = [];

    giveItem = addStrMine(GiveItems[k]);
    giveQuantity = GiveQuantitys[k];
    let haveQuantity = 0;
    let maxAmount = 64;
    let nonMax = 0;
    

    for(let i = 0; i < 36; i++) {
      if (!container.getItem(i)) {
        EmptySlot.push(i);
      } else if (container.getItem(i).typeId == giveItem) {
        haveQuantity += container.getItem(i).amount;
        maxAmount = container.getItem(i).maxAmount;
        if (maxAmount - haveQuantity >= giveQuantity) {
          nonMax += 1;
        }
      }
    }

    if (EmptySlot.length < GiveItems.length) { //スロット空き
      if (nonMax == 0) { //単体空き
        if (haveQuantity % maxAmount > (maxAmount - giveQuantity) || haveQuantity % maxAmount === 0) { //合計空き
          addSlot += 1;
          if (addSlot > EmptySlot.length) {
            player.sendMessage(`§c持ち物に空きがありません！`);
            return false;
          }
        }
      }
    }
  }
  return true;
}
